#include <assert.h>
#include <string.h>
#include <setjmp.h>

#define XOXA_PRIVATE_INTERFACE 1
#include "xml_expat.h"
#include "util.h"
#include "digest.h"
#include "pgp.h"
#undef XOXA_PRIVATE_INTERFACE

/*
 * The function xoxa_digest_initialize is
 * defined in per-implementation libraries.
 */

int xoxa_parse_for_digest(ParserState* parser,
                          const char* digest_name)
{
    int parse_status;

    if (strcmp(digest_name, "_") == 0) {
        if (SHOWING_MESSAGES(parser)) {
            fprintf(stderr, "Scanning for digest PI\n");
        }
        xoxa_push_output_callback(parser, &xoxa_null_callback, NULL, "null-callback");

    } else if (strcmp(digest_name, "pgp") == 0) {

        // PGP verification
        xoxa_pgp_verify_initialize(parser);

    } else {
        // xoxa_digest_initialize will do a longjmp if the digest name
        // is not recognised, so cope with that
        jmp_buf jmp_env;
        byte is_ok;

        if (setjmp(jmp_env) == 0) {
            xoxa_jmp_set_(parser, &jmp_env);

            xoxa_digest_initialize(parser, digest_name);
            is_ok = 1;
        } else {
            is_ok = 0;
        }
        xoxa_jmp_clear_(parser);
        if (! is_ok) {
            return 1;           // JUMP OUT
        }
    }

    parse_status = xoxa_parse(parser);

    return parse_status;        // non-zero on error
}

int xoxa_digest_is_initialized(ParserState* parser)
{
    if (parser != NULL && parser->d.digest.ctx) {
        return 1;
    } else {
        return 0;
    }
}
