// This is sample code illustrating the use of the Xoxa library.
//
// Perform an XML identity transformation, and simultaneously
// calculate the SHA-1 message digest of the input.
//
// Usage:
//
//    % xoxa_digest file.xml

#include <stdio.h>

#include "xoxa.h"

int main(int argc, char** argv)
{
    ParserState* PS;
    InputSource* src;
    int parse_status;
    const char* filename;
    int exit_status = 0;


    if (argc != 2) {
        fprintf(stderr, "Usage: %s <input.xml>\n", argv[0]);
        exit(1);
    }
    filename = argv[1];

    src = xoxa_new_input_source_from_filename(filename);
    if (src == NULL) {
        fprintf(stderr, "Can't open file %s to read\n", filename);
        exit(1);
    }
    PS = xoxa_new_parser(src);

    parse_status = xoxa_parse_for_digest(PS, "sha1");

    if (parse_status == 0) {
        size_t digest_len;
        int i;
        byte* digest = xoxa_get_digest(PS, &digest_len);

        printf("sha1(xoxa(%s))= ", filename);
        for (i=0; i<digest_len; i++) {
            printf("%02x", digest[i]);
        }
        printf("\n");

        xoxa_release_parser(PS);
    } else {
        fprintf(stderr, "Unable to parse file %s\n", filename);
        exit_status = 1;
    }

    return exit_status;
}
