/*
 * The following is the simple expat example code from the well-known
 * introduction to Expat at
 * <http://www.xml.com/lpt/a/1999/09/expat/index.html>.  The only
 * changes, beyond a couple of minor changes required to make it build
 * without warnings, are to replace all of the XML_* functions with
 * their analogous Xoxa_* functions, using '#define' macros, and to
 * print out at the end the accumulated digest of the input file.
 */

/*****************************************************************
 * outline.c
 *
 * Copyright 1999, Clark Cooper
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the same terms as Perl.
 *
 * Read an XML document from standard input and print an element
 * outline on standard output.
 */


#include <stdio.h>

#define WITH_XOXA 1
#if WITH_XOXA
#include "xoxa_expat.h"
#define XML_Parser Xoxa_Parser
//#define XML_ParserCreate(ENCODING) Xoxa_ParserCreateNS((ENCODING), '\xff')
#define XML_ParserCreate Xoxa_ParserCreate
#define XML_SetElementHandler Xoxa_SetElementHandler
#define XML_Parse Xoxa_Parse
#define XML_GetCurrentLineNumber Xoxa_GetCurrentLineNumber
#define XML_ErrorString Xoxa_ErrorString
#define XML_GetErrorCode Xoxa_GetErrorCode
#else
#include "expat.h"
#endif

#define BUFFSIZE	8192

char Buff[BUFFSIZE];

int Depth;

void
start(void *data, const char *el, const char **attr) {
  int i;

  for (i = 0; i < Depth; i++)
    printf("  ");

  printf("%s", el);

  for (i = 0; attr[i]; i += 2) {
    printf(" %s='%s'", attr[i], attr[i + 1]);
  }

  printf("\n");
  Depth++;
}  /* End of start handler */

void
end(void *data, const char *el) {
  Depth--;
}  /* End of end handler */

int
main(int argc, char **argv) {
  XML_Parser p = XML_ParserCreate(NULL);
  if (! p) {
    fprintf(stderr, "Couldn't allocate memory for parser\n");
    exit(-1);
  }

  XML_SetElementHandler(p, start, end);

  for (;;) {
    int done;
    int len;

    len = fread(Buff, 1, BUFFSIZE, stdin);
    if (ferror(stdin)) {
      fprintf(stderr, "Read error\n");
      exit(-1);
    }
    done = feof(stdin);

    if (! XML_Parse(p, Buff, len, done)) {
      fprintf(stderr, "Parse error at line %lu:\n%s\n",
	      XML_GetCurrentLineNumber(p),
	      XML_ErrorString(XML_GetErrorCode(p)));
      exit(-1);
    }

    if (done)
      break;
  }

#if WITH_XOXA
  {
    byte* md;
    size_t digest_length;
    int i;

    // extract the digest from the parser
    // (this remains owned by p)
    md = xoxa_get_digest(p, &digest_length);

    // and print it out
    fprintf(stderr, "digest: ");
    for (i=0; i<digest_length; i++) {
      fprintf(stderr, "%02x", md[i]);
    }
    fprintf(stderr, "\n");
  }
#endif
}  /* End of main */
