#ifndef PGP_TYPES_H_READ
#define PGP_TYPES_H_READ 1

#include <sys/select.h>

#include "util_types.h"

// This header contains types which are used in pgp.h, and which are
// also used in xml_expat.h, which pgp.h includes.  Separate them out
// here to avoid an import loop.

#if XOXA_PRIVATE_INTERFACE
// A channel for communicating with the subprocess.  Note that the
// Channel is a read or write channel _from the point of view of the
// parent process_ rather than the child.
//
// Creating a writing channel (non-zero argument to new_channel)
// creates a Buf within the channel; creating a reading channel does
// not, and one must be added to the channel subsequently.
typedef struct {
    int fd[2];                  // fd[0]=read end; fd[1]=write end
    Buf* buf;                   // the content to be written, or into which the pipe will be read
    ssize_t nwritten;           // negative if this channel is to be read from
} Channel;
#define CHANNEL_FOR_WRITING_P(ch) ((ch)->nwritten >= 0)
// Retrieve the FD associated with this Channel; this will be open for
// reading (respectively writing) if the Channel is a read (write) one.
#define CHANNEL_GET_FD(ch)        (CHANNEL_FOR_WRITING_P(ch) ? (ch)->fd[1] : (ch)->fd[0])
// Get the _other_ FD, which is the FD associated with the other
// end of the Channel's pipe, in the child.
#define CHANNEL_GET_CHILD_FD(ch)  (CHANNEL_FOR_WRITING_P(ch) ? (ch)->fd[0] : (ch)->fd[1])

typedef struct process_manager_s {
    int in;                     // the write end of a pipe, which goes to the subprocess stdin
    char nchan;                 // the number of extra channels (including the subprocess stdout/stderr)
    Channel** ch;               // the other channels
    pid_t child;                // the PID of the subprocess
    fd_set readable;            // a mask of the readable channels
} ProcessManager;
#endif /* XOXA_PRIVATE_INTERFACE */

/** The possible results of a PGP verification action */
typedef enum { good_sig, bad_sig, unknown } GPGResult;

/**
 * The SignatureDetail structure represents the results of a PGP
 * verification action.  If the status is `good_sig` or `bad_sig`,
 * then there are further details available in the corresponding
 * elements of the `detail` union.
 */
typedef struct gpg_result_s {
    /** The summary result of the signature verification */
    GPGResult status;
    /** Further details about the signature verification */
    union {
        /** Details in the case of a successful verification */
        struct {
            /** The hex identifier of the key used to make the signature */
            char* keyid;
            /** The identifier of the signer */
            char* username;
        } goodsig;
        /** Details in the case of an unsuccessful verification */
        struct {
            /** The hex identifier of the key used to make the signature */
            char* keyid;
            /** The identifier of the signer */
            char* username;
        } badsig;
    } detail;
    /** The contents of any text written to stderr by the PGP process */
    const char* stderr;         // content of GPG stderr, or NULL if unavailable
} SignatureDetail;


#endif /* PGP_TYPES_H_READ */
