/* GENERATED FILE -- DO NOT EDIT */

/* This file is part of the Xoxa library: https://bitbucket.org/nxg/xoxa
 * Copyright 2013, 2014, Norman Gray <http://nxg.me.uk>.
 * It is distributed under the terms of the BSD 2-clause Licence;
 * see the file LICENCE for details.
 */

#define XOXA_PRIVATE_INTERFACE 1
#include "xoxa_expat.h"
#include "util.h"
#undef XOXA_PRIVATE_INTERFACE

#include <assert.h>

/**
 * @file xoxa_expat.h
 * This module provides direct analogues of the
 * <a href='http://www.libexpat.org'>Expat</a> `XML_*` functions.
 * Each of the `Xoxa_*` functions below should be used in
 * the same way as the corresponding `XML_*` function,
 * except that the first argument is of type {@link Xoxa_Parser} rather
 * than Expat's <code>XML_Parser</code>.
 *
 * @section expat Wrapping Expat
 *
 * The library also provides a set of functions `Xoxa_...`
 * which precisely mirror the `XML_...` functions of the Expat
 * library, and which can be used in an analogous way.  The intention
 * is that the XML_* functions can be directly replaced by the Xoxa_*
 * functions (for example with `#define XML_ParserCreate
 * Xoxa_ParserCreate`) with no other code changes.
 *
 *     #include "xoxa_expat.h"
 *
 *     [...]
 *     Xoxa_Parser xoxa_p = Xoxa_ParserCreateNS(NULL);
 *     void* digest_ctx;
 *
 *     // Install element and content handlers, just as with Expat
 *     Xoxa_SetElementHandler(xoxa_p, start_element_handler, end_element_handler);
 *     Xoxa_SetCharacterDataHandler(xoxa_p, characterdata_handler);
 *     Xoxa_SetUserData(xoxa_p, (void*)&S);
 *
 *     // Initialise the Xoxa digest
 *     digest_ctx = xoxa_digest_initialize(xoxa_p, "SHA1");
 *
 *     [...]
 *     // Process the file (the installed handlers are called as usual)
 *     buf = Xoxa_GetBuffer(xoxa_p, BUFSIZ);
 *     Xoxa_ParseBuffer(xoxa_p, bytes_read, is_eof);
 *     [...]
 *
 *     byte* md;
 *     size_t digest_length;
 *     // use a Xoxa function to extract the message digest.
 *     // (the contents of this buffer are owned by xoxa_p)
 *     md = xoxa_get_digest(xoxa_p, &digest_length);
 *     [...print out md...]
 *
 *     Xoxa_ParserFree(xoxa_p);
 *
 * The `Xoxa_...` functions require namespace-aware parsing.
 * Therefore the `Xoxa_ParserCreate` and `Xoxa_ParserCreate_MM`
 * functions, if called, emit a warning and call the equivalent of
 * `Xoxa_ParserCreateNS` instead.  The handler functions which are
 * passed to the library should be written with this in mind.
 *
 * The Xoxa-C library supports both message-digest
 * functions.  You can call either or both of `xoxa_get_digest` and
 * `xoxa_get_signature` on the result.
 */

/*

*/

Xoxa_Parser Xoxa_ParserCreate(const XML_Char *encoding)
{
    //warning("Xoxa_ParserCreate called; ParserCreateNS called instead");
    return xoxa_new_parser(NULL);
}

Xoxa_Parser Xoxa_ParserCreateNS(const XML_Char *encoding, XML_Char namespaceSeparator)
{
    return xoxa_new_parser(NULL);
}

Xoxa_Parser Xoxa_ParserCreate_MM(const XML_Char *encoding, const XML_Memory_Handling_Suite *memsuite, const XML_Char *namespaceSeparator)
{
    //warning("Xoxa_ParserCreate_MM called; ParserCreateNS called instead");
    return xoxa_new_parser(NULL);
}

Xoxa_Parser Xoxa_ExternalEntityParserCreate(Xoxa_Parser xoxa_parser, const XML_Char* context, const XML_Char* encoding)
{
    XML_Parser xml_parser;
    XML_Parser xml_subparser;
    Xoxa_Parser xoxa_subparser;

    if (xoxa_parser == NULL || !PARSERSTATE_OK_TEST_(xoxa_parser)) {
        warning("bad or missing parser given to Xoxa_ExternalEntityParserCreate");
        return NULL;
    }
    xml_parser = xoxa_parser->parser;
    xml_subparser = XML_ExternalEntityParserCreate(xml_parser, context, encoding);
    xoxa_subparser = xoxa_make_new_parserstate_(xoxa_parser->src, xml_subparser, xoxa_parser);

    return xoxa_subparser;
}

const XML_LChar * Xoxa_ErrorString(enum XML_Error code)
{
    return XML_ErrorString(code);
}

const XML_LChar * Xoxa_ExpatVersion(void)
{
    return XML_ExpatVersion();
}

XML_Expat_Version Xoxa_ExpatVersionInfo(void)
{
    return XML_ExpatVersionInfo();
}

const XML_Feature * Xoxa_GetFeatureList(void)
{
    return XML_GetFeatureList();
}

// The extra-handler functions here are additional callbacks for the Expat functions
// which xml_expat.c itself overrides.  The list of functions here must match the list of
// such initialisations in xml_expat.c:xoxa_new_parser.


void Xoxa_SetXmlDeclHandler(Xoxa_Parser xp, XML_XmlDeclHandler arg)
{
    xp->extra_XmlDeclHandler = arg;
}

void Xoxa_SetStartElementHandler(Xoxa_Parser xp, XML_StartElementHandler arg)
{
    xp->extra_StartElementHandler = arg;
}
void Xoxa_SetEndElementHandler(Xoxa_Parser xp, XML_EndElementHandler arg)
{
    xp->extra_EndElementHandler = arg;
}
void Xoxa_SetElementHandler(Xoxa_Parser xp, XML_StartElementHandler arg2, XML_EndElementHandler arg1)
{
    Xoxa_SetStartElementHandler(xp, arg2);
    Xoxa_SetEndElementHandler(xp, arg1);
}

void Xoxa_SetStartCdataSectionHandler(Xoxa_Parser xp, XML_StartCdataSectionHandler arg)
{
    xp->extra_StartCdataSectionHandler = arg;
}
void Xoxa_SetEndCdataSectionHandler(Xoxa_Parser xp, XML_EndCdataSectionHandler arg)
{
    xp->extra_EndCdataSectionHandler = arg;
}
void Xoxa_SetCdataSectionHandler(Xoxa_Parser xp, XML_StartCdataSectionHandler arg2, XML_EndCdataSectionHandler arg1)
{
    Xoxa_SetStartCdataSectionHandler(xp, arg2);
   Xoxa_SetEndCdataSectionHandler(xp, arg1);
}

void Xoxa_SetStartNamespaceDeclHandler(Xoxa_Parser xp, XML_StartNamespaceDeclHandler arg)
{
    xp->extra_StartNamespaceDeclHandler = arg;
}
void Xoxa_SetEndNamespaceDeclHandler(Xoxa_Parser xp, XML_EndNamespaceDeclHandler arg)
{
    xp->extra_EndNamespaceDeclHandler = arg;
}
void Xoxa_SetNamespaceDeclHandler(Xoxa_Parser xp, XML_StartNamespaceDeclHandler arg2, XML_EndNamespaceDeclHandler arg1)
{
    Xoxa_SetStartNamespaceDeclHandler(xp, arg2);
    Xoxa_SetEndNamespaceDeclHandler(xp, arg1);
}

void Xoxa_SetCharacterDataHandler(Xoxa_Parser xp, XML_CharacterDataHandler arg)
{
    xp->extra_CharacterDataHandler = arg;
}
void Xoxa_SetProcessingInstructionHandler(Xoxa_Parser xp, XML_ProcessingInstructionHandler arg)
{
    xp->extra_ProcessingInstructionHandler = arg;
}


void Xoxa_SetExternalEntityRefHandler(Xoxa_Parser xoxa_parser, Xoxa_ExternalEntityRefHandler handler)
{
    if (xoxa_parser == NULL || !PARSERSTATE_OK_TEST_(xoxa_parser)) {
        warning("bad or missing parser given to Xoxa_ExternalEntityParserCreate");
        return;
    }

    XML_SetExternalEntityRefHandler(xoxa_parser->parser,
                                    (XML_ExternalEntityRefHandler)handler);
    // Unless Xoxa_SetExternalEntityRefHandlerArg is subsequently called,
    // the object passed to the handler is the Xoxa_Parser.
    XML_SetExternalEntityRefHandlerArg(xoxa_parser->parser,
                                       xoxa_parser);
}
void Xoxa_SetExternalEntityRefHandlerArg(Xoxa_Parser xoxa_parser, void* arg1)
{
    if (xoxa_parser == NULL || !PARSERSTATE_OK_TEST_(xoxa_parser)) {
        warning("bad or missing parser given to Xoxa_ExternalEntityParserCreate");
        return;
    }

    if (arg1 != NULL) {
        // The docs for XML_SetExternalEntityRefHandlerArg say "If arg is not NULL,
        // it is the new value passed to the handler set using
        // XML_SetExternalEntityRefHandler; if arg is NULL, the argument passed
        // to the handler function will be the parser object itself."
        // So imitate this.  If this is not called, then the Xoxa_Parser is passed
        // to the handler (cf Xoxa_SetExternalEntityRefHandler above).
        XML_SetExternalEntityRefHandlerArg(xoxa_parser->parser, arg1);
    }
}
void Xoxa_SetSkippedEntityHandler(Xoxa_Parser xp, XML_SkippedEntityHandler arg)
{
    xp->extra_SkippedEntityHandler = arg;
}

void Xoxa_SetCommentHandler(Xoxa_Parser xp, XML_CommentHandler arg1)
{
    return XML_SetCommentHandler(xp->parser, arg1);
}
void Xoxa_SetDefaultHandler(Xoxa_Parser xp, XML_DefaultHandler arg1)
{
    return XML_SetDefaultHandler(xp->parser, arg1);
}
void Xoxa_SetDefaultHandlerExpand(Xoxa_Parser xp, XML_DefaultHandler arg1)
{
    return XML_SetDefaultHandlerExpand(xp->parser, arg1);
}

XML_Bool Xoxa_ParserReset(Xoxa_Parser xp, const XML_Char* arg1)
{
    return XML_ParserReset(xp->parser, arg1);
}
void Xoxa_SetEntityDeclHandler(Xoxa_Parser xp, XML_EntityDeclHandler arg1)
{
    return XML_SetEntityDeclHandler(xp->parser, arg1);
}
void Xoxa_SetElementDeclHandler(Xoxa_Parser xp, XML_ElementDeclHandler arg1)
{
    return XML_SetElementDeclHandler(xp->parser, arg1);
}
void Xoxa_SetAttlistDeclHandler(Xoxa_Parser xp, XML_AttlistDeclHandler arg1)
{
    return XML_SetAttlistDeclHandler(xp->parser, arg1);
}
void Xoxa_SetDoctypeDeclHandler(Xoxa_Parser xp, XML_StartDoctypeDeclHandler arg2, XML_EndDoctypeDeclHandler arg1)
{
    return XML_SetDoctypeDeclHandler(xp->parser, arg2, arg1);
}
void Xoxa_SetStartDoctypeDeclHandler(Xoxa_Parser xp, XML_StartDoctypeDeclHandler arg1)
{
    return XML_SetStartDoctypeDeclHandler(xp->parser, arg1);
}
void Xoxa_SetEndDoctypeDeclHandler(Xoxa_Parser xp, XML_EndDoctypeDeclHandler arg1)
{
    return XML_SetEndDoctypeDeclHandler(xp->parser, arg1);
}
void Xoxa_SetUnparsedEntityDeclHandler(Xoxa_Parser xp, XML_UnparsedEntityDeclHandler arg1)
{
    return XML_SetUnparsedEntityDeclHandler(xp->parser, arg1);
}
void Xoxa_SetNotationDeclHandler(Xoxa_Parser xp, XML_NotationDeclHandler arg1)
{
    return XML_SetNotationDeclHandler(xp->parser, arg1);
}

void Xoxa_SetNotStandaloneHandler(Xoxa_Parser xp, XML_NotStandaloneHandler arg1)
{
    return XML_SetNotStandaloneHandler(xp->parser, arg1);
}
void Xoxa_SetUnknownEncodingHandler(Xoxa_Parser xp, XML_UnknownEncodingHandler arg2, void * arg1)
{
    return XML_SetUnknownEncodingHandler(xp->parser, arg2, arg1);
}
void Xoxa_DefaultCurrent(Xoxa_Parser xp)
{
    return XML_DefaultCurrent(xp->parser);
}
void Xoxa_SetReturnNSTriplet(Xoxa_Parser xp, int arg1)
{
    return XML_SetReturnNSTriplet(xp->parser, arg1);
}
void Xoxa_SetUserData(Xoxa_Parser xp, void * arg1)
{
    xp->extra_UserData = arg1;
}
enum XML_Status Xoxa_SetEncoding(Xoxa_Parser xp, const XML_Char * arg1)
{
    return XML_SetEncoding(xp->parser, arg1);
}
void Xoxa_UseParserAsHandlerArg(Xoxa_Parser xp)
{
    return XML_UseParserAsHandlerArg(xp->parser);
}
enum XML_Error Xoxa_UseForeignDTD(Xoxa_Parser xp, XML_Bool arg1)
{
    return XML_UseForeignDTD(xp->parser, arg1);
}
enum XML_Status Xoxa_SetBase(Xoxa_Parser xp, const XML_Char * arg1)
{
    return XML_SetBase(xp->parser, arg1);
}
const XML_Char * Xoxa_GetBase(Xoxa_Parser xp)
{
    return XML_GetBase(xp->parser);
}
int Xoxa_GetSpecifiedAttributeCount(Xoxa_Parser xp)
{
    return XML_GetSpecifiedAttributeCount(xp->parser);
}
int Xoxa_GetIdAttributeIndex(Xoxa_Parser xp)
{
    return XML_GetIdAttributeIndex(xp->parser);
}
enum XML_Status Xoxa_Parse(Xoxa_Parser xp, const char * arg3, int arg2, int arg1)
{
    return XML_Parse(xp->parser, arg3, arg2, arg1);
}
void * Xoxa_GetBuffer(Xoxa_Parser xp, int arg1)
{
    return XML_GetBuffer(xp->parser, arg1);
}
enum XML_Status Xoxa_ParseBuffer(Xoxa_Parser xp, int arg2, int arg1)
{
    return XML_ParseBuffer(xp->parser, arg2, arg1);
}
enum XML_Status Xoxa_StopParser(Xoxa_Parser xp, XML_Bool arg1)
{
    return XML_StopParser(xp->parser, arg1);
}
enum XML_Status Xoxa_ResumeParser(Xoxa_Parser xp)
{
    return XML_ResumeParser(xp->parser);
}
void Xoxa_GetParsingStatus(Xoxa_Parser xp, XML_ParsingStatus * arg1)
{
    return XML_GetParsingStatus(xp->parser, arg1);
}
int Xoxa_SetParamEntityParsing(Xoxa_Parser xp, enum XML_ParamEntityParsing arg1)
{
    return XML_SetParamEntityParsing(xp->parser, arg1);
}
enum XML_Error Xoxa_GetErrorCode(Xoxa_Parser xp)
{
    return XML_GetErrorCode(xp->parser);
}
XML_Size Xoxa_GetCurrentLineNumber(Xoxa_Parser xp)
{
    return XML_GetCurrentLineNumber(xp->parser);
}
XML_Size Xoxa_GetCurrentColumnNumber(Xoxa_Parser xp)
{
    return XML_GetCurrentColumnNumber(xp->parser);
}
XML_Index Xoxa_GetCurrentByteIndex(Xoxa_Parser xp)
{
    return XML_GetCurrentByteIndex(xp->parser);
}
int Xoxa_GetCurrentByteCount(Xoxa_Parser xp)
{
    return XML_GetCurrentByteCount(xp->parser);
}
const char * Xoxa_GetInputContext(Xoxa_Parser xp, int * arg2, int * arg1)
{
    return XML_GetInputContext(xp->parser, arg2, arg1);
}
void Xoxa_FreeContentModel(Xoxa_Parser xp, XML_Content * arg1)
{
    return XML_FreeContentModel(xp->parser, arg1);
}
void * Xoxa_MemMalloc(Xoxa_Parser xp, size_t arg1)
{
    return XML_MemMalloc(xp->parser, arg1);
}
void * Xoxa_MemRealloc(Xoxa_Parser xp, void * arg2, size_t arg1)
{
    return XML_MemRealloc(xp->parser, arg2, arg1);
}
void Xoxa_MemFree(Xoxa_Parser xp, void * arg1)
{
    return XML_MemFree(xp->parser, arg1);
}
void Xoxa_ParserFree(Xoxa_Parser P)
{
    xoxa_release_parser(P);
}




